/*
 +----------------------------------------------------------------------+
 | This file is part of the Ayyi project. https://ayyi.org              |
 | copyright (C) 2020-2022 Tim Orford <tim@orford.org>                  |
 +----------------------------------------------------------------------+
 | This program is free software; you can redistribute it and/or modify |
 | it under the terms of the GNU General Public License version 3       |
 | as published by the Free Software Foundation.                        |
 +----------------------------------------------------------------------+
 |
 */

#include "config.h"
#undef USE_GTK
#include <getopt.h>
#include "gdk/gdk.h"
#include "agl/x11.h"
#include "agl/debug.h"
#include "agl/text/text_node.h"
#include "actors/spinner.h"
#define __glx_test__
#include "test/common2.h"

extern AGlMaterialClass aaline_class;

#define N_LINES 16

static KeyHandler
	zoom_in,
	zoom_out;

Key keys[] = {
	{XK_equal,       zoom_in},
	{XK_KP_Add,      zoom_in},
	{XK_minus,       zoom_out},
	{XK_KP_Subtract, zoom_out},
	{XK_Escape,      (KeyHandler*)exit},
	{XK_q,           (KeyHandler*)exit},
};

static const struct option long_options[] = {
	{ "non-interactive",  0, NULL, 'n' },
};

static const char* const short_options = "n";


int
main (int argc, char* argv[])
{
	const int width = 400, height = 200;

	if (g_getenv("NON_INTERACTIVE")) {
		g_timeout_add(3000, (gpointer)exit, NULL);
	}

	int opt;
	while ((opt = getopt_long (argc, argv, short_options, long_options, NULL)) != -1) {
		switch (opt) {
			case 'n':
				g_timeout_add(3000, (gpointer)exit, NULL);
				break;
		}
	}

	AGlWindow* window = agl_window("Text test", 0, 0, width, height, 0);
	XMapWindow(dpy, window->window);

	AGlActor* text = agl_actor__add_child((AGlActor*)window->scene, text_node(NULL));
	text_node_set_text((TextNode*)text, g_strdup("Spinner test"));
	text->colour = 0xbbbbbbff;
	text->region = (AGlfRegion){.x2 = 80, .y2 = 30};

	AGlActor* spinner = agl_actor__add_child((AGlActor*)window->scene, agl_spinner(NULL));
	((AGlSpinner*)spinner)->spinning = true;

	add_key_handlers(keys);

	g_main_loop_run(agl_main_loop_new());

	agl_window_destroy(&window);
	XCloseDisplay(dpy);

	return 0;
}


static void
zoom_in (gpointer user_data)
{
}


static void
zoom_out (gpointer user_data)
{
}
